@@ -13,19 +13,30 @@
 //! - `hyper-h1`: hyper support with http/1
 //! - `hyper-h2`: hyper support with http/2
 //! - `tokio-net`: Implementations for tokio socket types (default)
+//! - `rt`: Features that depend on the tokio runtime, such as [`SpawningHandshakes`]
 
 use futures_util::stream::{FuturesUnordered, Stream, StreamExt};
 use pin_project_lite::pin_project;
+#[cfg(feature = "rt")]
+pub use spawning_handshake::SpawningHandshakes;
 use std::future::Future;
-use std::marker::Unpin;
 use std::pin::Pin;
 use std::task::{Context, Poll};
 use std::time::Duration;
-use std::{io, mem};
 use thiserror::Error;
 use tokio::io::{AsyncRead, AsyncWrite};
 use tokio::time::{timeout, Timeout};
 
+#[cfg(feature = "rt")]
+mod spawning_handshake;
+
+/// This module contains feature specific to integrating with the hyper library.
+#[cfg(any(feature = "hyper-h1", feature = "hyper-h2"))]
+pub mod hyper;
+
+#[cfg(feature = "tokio-net")]
+mod net;
+
 /// Default number of concurrent handshakes
 pub const DEFAULT_MAX_HANDSHAKES: usize = 64;
 /// Default timeout for the TLS handshake.
@@ -40,7 +51,7 @@ pub trait AsyncTls<C: AsyncRead + AsyncWrite>: Clone {
     /// Error type for completing the TLS handshake
     type Error: std::error::Error;
     /// Type of the Future for the TLS stream that is accepted.
-    type AcceptFuture: Future<Output = Result<Self::Stream, Self::Error>> + Unpin;
+    type AcceptFuture: Future<Output = Result<Self::Stream, Self::Error>>;
 
     /// Accept a TLS connection on an underlying stream
     fn accept(&self, stream: C) -> Self::AcceptFuture;
@@ -57,7 +68,24 @@ pub trait AsyncAccept {
     fn poll_accept(
         self: Pin<&mut Self>,
         cx: &mut Context<'_>,
-    ) -> Poll<Result<Self::Connection, Self::Error>>;
+    ) -> Poll<Option<Result<Self::Connection, Self::Error>>>;
+
+    /// Return a new `AsyncAccept` that stops accepting connections after
+    /// `ender` completes.
+    ///
+    /// Useful for graceful shutdown.
+    ///
+    /// See [examples/echo.rs](https://github.com/tmccombs/tls-listener/blob/main/examples/echo.rs)
+    /// for example of how to use.
+    fn until<F: Future>(self, ender: F) -> Until<Self, F>
+    where
+        Self: Sized,
+    {
+        Until {
+            acceptor: self,
+            ender,
+        }
+    }
 }
 
 pin_project! {
@@ -76,13 +104,18 @@ pin_project! {
     ///
     /// By default, if a client fails the TLS handshake, that is treated as an error, and the
     /// `TlsListener` will return an `Err`. If the `TlsListener` is passed directly to a hyper
-    /// `Server`, then an invalid handshake can cause the server to stop accepting connections.
-    /// See `http-stream.rs` or `http-low-level` examples, for examples of how to avoid this.
+    /// [`Server`][1], then an invalid handshake can cause the server to stop accepting connections.
+    /// See [`http-stream.rs`][2] or [`http-low-level`][3] examples, for examples of how to avoid this.
     ///
     /// Note that if the maximum number of pending connections is greater than 1, the resulting
-    /// `TlsStream` connections may come in a different order than the connections produced by the
+    /// [`T::Stream`][4] connections may come in a different order than the connections produced by the
     /// underlying listener.
     ///
+    /// [1]: https://docs.rs/hyper/latest/hyper/server/struct.Server.html
+    /// [2]: https://github.com/tmccombs/tls-listener/blob/main/examples/http-stream.rs
+    /// [3]: https://github.com/tmccombs/tls-listener/blob/main/examples/http-low-level.rs
+    /// [4]: AsyncTls::Stream
+    ///
     pub struct TlsListener<A: AsyncAccept, T: AsyncTls<A::Connection>> {
         #[pin]
         listener: A,
@@ -125,7 +158,6 @@ where
 impl<A, T> TlsListener<A, T>
 where
     A: AsyncAccept,
-    A::Connection: AsyncRead + AsyncWrite + Unpin + 'static,
     A::Error: std::error::Error,
     T: AsyncTls<A::Connection>,
 {
@@ -159,7 +191,6 @@ where
 impl<A, T> Stream for TlsListener<A, T>
 where
     A: AsyncAccept,
-    A::Connection: AsyncRead + AsyncWrite + Unpin + 'static,
     A::Error: std::error::Error,
     T: AsyncTls<A::Connection>,
 {
@@ -171,13 +202,14 @@ where
         while this.waiting.len() < *this.max_handshakes {
             match this.listener.as_mut().poll_accept(cx) {
                 Poll::Pending => break,
-                Poll::Ready(Ok(conn)) => {
+                Poll::Ready(Some(Ok(conn))) => {
                     this.waiting
                         .push(timeout(*this.timeout, this.tls.accept(conn)));
                 }
-                Poll::Ready(Err(e)) => {
+                Poll::Ready(Some(Err(e))) => {
                     return Poll::Ready(Some(Err(Error::ListenerError(e))));
                 }
+                Poll::Ready(None) => return Poll::Ready(None),
             }
         }
 
@@ -198,7 +230,7 @@ where
 #[cfg(feature = "rustls")]
 impl<C: AsyncRead + AsyncWrite + Unpin> AsyncTls<C> for tokio_rustls::TlsAcceptor {
     type Stream = tokio_rustls::server::TlsStream<C>;
-    type Error = io::Error;
+    type Error = std::io::Error;
     type AcceptFuture = tokio_rustls::Accept<C>;
 
     fn accept(&self, conn: C) -> Self::AcceptFuture {
@@ -275,83 +307,29 @@ pub fn builder<T>(tls: T) -> Builder<T> {
     }
 }
 
-#[cfg(feature = "tokio-net")]
-#[cfg_attr(docsrs, doc(cfg(feature = "tokio-net")))]
-impl AsyncAccept for tokio::net::TcpListener {
-    type Connection = tokio::net::TcpStream;
-    type Error = io::Error;
+pin_project! {
+    /// See [`AsyncAccept::until`]
+    pub struct Until<A, E> {
+        #[pin]
+        acceptor: A,
+        #[pin]
+        ender: E,
+    }
+}
+
+impl<A: AsyncAccept, E: Future> AsyncAccept for Until<A, E> {
+    type Connection = A::Connection;
+    type Error = A::Error;
 
     fn poll_accept(
         self: Pin<&mut Self>,
         cx: &mut Context<'_>,
-    ) -> Poll<Result<Self::Connection, Self::Error>> {
-        match (*self).poll_accept(cx) {
-            Poll::Ready(Ok((stream, _))) => Poll::Ready(Ok(stream)),
-            Poll::Ready(Err(e)) => Poll::Ready(Err(e)),
-            Poll::Pending => Poll::Pending,
-        }
-    }
-}
+    ) -> Poll<Option<Result<Self::Connection, Self::Error>>> {
+        let this = self.project();
 
-#[cfg(all(unix, feature = "tokio-net"))]
-#[cfg_attr(docsrs, doc(cfg(feature = "tokio-net")))]
-impl AsyncAccept for tokio::net::UnixListener {
-    type Connection = tokio::net::UnixStream;
-    type Error = io::Error;
-
-    fn poll_accept(
-        self: Pin<&mut Self>,
-        cx: &mut Context<'_>,
-    ) -> Poll<Result<Self::Connection, Self::Error>> {
-        match (*self).poll_accept(cx) {
-            Poll::Ready(Ok((stream, _))) => Poll::Ready(Ok(stream)),
-            Poll::Ready(Err(e)) => Poll::Ready(Err(e)),
-            Poll::Pending => Poll::Pending,
-        }
-    }
-}
-
-// Possibly make a blanket implementation for hyper::server::accept::Accept?
-
-#[cfg(any(feature = "hyper-h1", feature = "hyper-h2"))]
-mod hyper_impl {
-    use super::*;
-    use hyper::server::accept::Accept as HyperAccept;
-    use hyper::server::conn::{AddrIncoming, AddrStream};
-
-    #[cfg_attr(docsrs, doc(cfg(any(feature = "hyper-h1", feature = "hyper-h2"))))]
-    impl AsyncAccept for AddrIncoming {
-        type Connection = AddrStream;
-        type Error = io::Error;
-
-        fn poll_accept(
-            self: Pin<&mut Self>,
-            cx: &mut Context<'_>,
-        ) -> Poll<Result<Self::Connection, Self::Error>> {
-            match <AddrIncoming as HyperAccept>::poll_accept(self, cx) {
-                Poll::Ready(Some(res)) => Poll::Ready(res),
-                Poll::Pending => Poll::Pending,
-                Poll::Ready(None) => unreachable!("None returned from AddrIncoming"),
-            }
-        }
-    }
-
-    #[cfg_attr(docsrs, doc(cfg(any(feature = "hyper-h1", feature = "hyper-h2"))))]
-    impl<A, T> HyperAccept for TlsListener<A, T>
-    where
-        A: AsyncAccept,
-        A::Connection: AsyncRead + AsyncWrite + Unpin + 'static,
-        A::Error: std::error::Error,
-        T: AsyncTls<A::Connection>,
-    {
-        type Conn = T::Stream;
-        type Error = Error<A::Error, T::Error>;
-
-        fn poll_accept(
-            self: Pin<&mut Self>,
-            cx: &mut Context<'_>,
-        ) -> Poll<Option<Result<Self::Conn, Self::Error>>> {
-            self.poll_next(cx)
+        match this.ender.poll(cx) {
+            Poll::Pending => this.acceptor.poll_accept(cx),
+            Poll::Ready(_) => Poll::Ready(None),
         }
     }
 }
